#pragma once

#include "data/bonus_points.hpp"
#include "data/nodes/dynexite_exam.hpp"
#include "data/nodes/student_data.hpp"
#include "util/grade.hpp"
#include "util/grade_boundary_step.hpp"

#include <array>
#include <filesystem>
#include <memory>
#include <string>

struct CsvDump;

namespace impl {

/**
 * \brief Converts all reasonable string representations of a bool to a bool.
 */
[[nodiscard]] bool toBool(std::string str);
} // namespace impl

struct YmlSettings {
    std::string mVersion{};

    GradeBoundaryStepSize mUserPointConfigStepSize =
        GradeBoundaryStepSize::Min1_00;
    bool mUserPointConfigCountZero{};

    bool mUserGradeConfigRequirePassAll{};

    int mChangedMaxPointsClosed{};
    bool mChangedMaxPointsClosedFollowDefault{};

    int mChangedMaxPointsOpen{};
    bool mChangedMaxPointsOpenFollowDefault{};

    bool mUserGradeBoundariesClosedAuto{};
    std::array<int, grades.size()> mUserGradeBoundariesClosed{};

    bool mUserGradeBoundariesOpenAuto{};
    std::array<int, grades.size()> mUserGradeBoundariesOpen{};
};

/**
 * \brief Contains all data loaded from a yml file
 */
struct ExamSave {
    ExamSave() = default;
    ExamSave(const CsvDump &dynexite, CsvDump &rwthOnline, const CsvDump &bonus,
             const YmlSettings &settings);

    DynexiteExam mDynexite;
    StudentData mStudents;
    BonusPoints mBonus;
    YmlSettings mSettings;
};

/**
 * \brief Reads the file at the given paths into \ref ExamSave if possible.
 */
std::unique_ptr<ExamSave> readYml(const std::filesystem::path &p);

/**
 * \brief Creates a file at the given location and writes the current settings
 * to it.
 *
 * Overwrites anything previously stored in the file.
 */
void writeYml(const std::filesystem::path &p);
