#include "statistic_window.hpp"

#include "data/data_manager.hpp"

#include <QLocale>

#include <algorithm>
#include <iterator>
#include <vector>

StatisticWindow::StatisticWindow(QWidget *parent)
    : QWidget(parent), mLayout(this) {

    setWindowTitle("Statistics");

    mLayout.addWidget(new QLabel("Attended students:", this), 0, 0);
    mLayout.addWidget(new QLabel("\tRegistered:", this), 1, 0);
    mLayout.addWidget(new QLabel("\tNot registered:", this), 2, 0);
    mLayout.addWidget(new QLabel("Registered students:", this), 3, 0);
    mLayout.addWidget(new QLabel("\tAttended:", this), 4, 0);
    mLayout.addWidget(new QLabel("\tNot attended:", this), 5, 0);

    mLayout.addWidget(&mNrAttended, 0, 1);
    mLayout.addWidget(&mNrAttendedRegistered, 1, 1);
    mLayout.addWidget(&mNrAttendedNotRegistered, 2, 1);
    mLayout.addWidget(&mNrRegistered, 3, 1);
    mLayout.addWidget(&mNrRegisteredAttended, 4, 1);
    mLayout.addWidget(&mNrRegisteredNotAttended, 5, 1);

    connect(&DataManager::getInstance(), &DataManager::rawDataChanged, this,
            &StatisticWindow::pointsChanged);
    connect(&DataManager::getInstance(), &DataManager::gradesChanged, this,
            &StatisticWindow::gradesChanged);
}

void StatisticWindow::pointsChanged() {

    const auto cur = DataManager::getCurrent();
    if (!cur) {
        mNrAttended.setText("-");
        mNrAttendedRegistered.setText("-");
        mNrAttendedNotRegistered.setText("-");
        mNrRegistered.setText("-");
        mNrRegisteredAttended.setText("-");
        mNrRegisteredNotAttended.setText("-");
        return;
    }

    // Find number of students who are registered and attended
    // Dynexite lists attended students,
    // RWTHonline lists registered students
    auto attendedStudents = cur->mExam.mIdentifier;
    auto registeredStudents = cur->mStudents.mRegistrationNr;

    std::ranges::sort(attendedStudents);
    std::ranges::sort(registeredStudents);

    std::vector<int> intersection;
    std::ranges::set_intersection(registeredStudents, attendedStudents,
                                  std::back_inserter(intersection));

    const auto nrAttended = attendedStudents.size();
    const auto nrRegistered = registeredStudents.size();
    const auto nrRegisteredAttended = static_cast<qsizetype>(intersection.size());

    mNrAttended.setText(QLocale().toString(nrAttended));
    mNrAttendedRegistered.setText(QLocale().toString(nrRegisteredAttended));
    mNrAttendedNotRegistered.setText(
        QLocale().toString(nrAttended - nrRegisteredAttended));
    mNrRegistered.setText(QLocale().toString(nrRegistered));
    mNrRegisteredAttended.setText(QLocale().toString(nrRegisteredAttended));
    mNrRegisteredNotAttended.setText(
        QLocale().toString(nrRegistered - nrRegisteredAttended));
}

void StatisticWindow::gradesChanged() {}
